@@ -1,6 +1,8 @@
 # Dots - A braille translation program.
 #
 # Copyright (C) 2009 Eitan Isaacson
+# Copyright (C) 2010 Consorcio Fernando de los Rios
+#               Author: Fernando Herrera <fherrera@onirica.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,11 +18,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import os, tempfile, host_settings, sys, gtk
 import ascii_braille
-import gtksourceview2, pango
+import gtksourceview2, pango, gobject
 import mimetypes
 
 class DotsProject(gtk.ScrolledWindow):
-    def __init__(self, input_file, name):
+    def __init__(self, document, name):
         gtk.ScrolledWindow.__init__(self)
         self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         self.view = gtksourceview2.View()
@@ -28,15 +30,21 @@ class DotsProject(gtk.ScrolledWindow):
         self.view.set_left_margin(25)
         self.view.set_editable(False)
         self.view.modify_font(pango.FontDescription('Mono'))
+	self.view.connect_after("move-cursor", self._onMoved)
         self.add(self.view)
         self.buffer = self.view.get_buffer()
         self.buffer.connect("modified-changed", self._onModified)
         self.braille_buffer = gtksourceview2.Buffer()
-        self.input_file = input_file
+        self.document = document
         self.tab_label = gtk.Label()
         self.set_name(name)
         self.out_file = None
-        self.config_text = None
+        self.config = None
+	self.current_line = None
+	try:
+		gobject.signal_new("line-changed", self, gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gtk.ScrolledWindow, gobject.TYPE_STRING))
+	except:
+		pass
 
     def set_name(self, name):
         gtk.ScrolledWindow.set_name(self, name)
@@ -48,9 +56,24 @@ class DotsProject(gtk.ScrolledWindow):
         else:
             self.tab_label.set_text(self.name)
 
-    def transcribeBraille(self, config_text):
-        self.config_text = config_text
-        self._transcribeBraille()
+    def _onMoved(self, view, step, count, extend_selection):
+	mark = view.get_buffer().get_mark("insert")
+	iter = view.get_buffer().get_iter_at_mark(mark)
+	line = iter.get_line()
+	start = self.buffer.get_iter_at_line(line)
+	end = start.copy()
+	end.forward_to_line_end()
+	text = self.buffer.get_text (start, end)
+	if text != self.current_line:
+		self.current_line = text
+		self.emit("line-changed", self, self.current_line)
+
+		
+	
+
+    def transcribeBraille(self, config):
+        self.config = config
+        return self._transcribeBraille()
 
     def view_ascii(self):
         self.view.set_buffer(self.buffer)
@@ -59,35 +82,24 @@ class DotsProject(gtk.ScrolledWindow):
         self.view.set_buffer(self.braille_buffer)
 
     def _transcribeBraille(self):
-        # Write config file.
-        fd, config_fn = tempfile.mkstemp('.config','dots_')
-        fconfig = os.fdopen(fd, 'w')
-        fconfig.write(self.config_text)
-        fconfig.close()
+	self.document.translate (self.config)
+	
+        braille_text = self.document.get_braille_text() 
+	if braille_text == "":
+		return False
 
-        # Create a temporary out file.
-        outfile = tempfile.mktemp('.brl', 'dots_')
 
-        mimetype, ignore = mimetypes.guess_type(self.input_file)
-        if mimetype == 'application/msword':
-            argv = '%s -x db "%s" | %s -f %s > "%s"' % \
-                (host_settings.antiword, self.input_file, 
-                 host_settings.xml2brl, config_fn, outfile)
-        else:
-            argv = '%s -f %s < %s > "%s"' % \
-                (host_settings.xml2brl, config_fn, self.input_file, outfile)
+	# Workaround for strange liblouis bug
+	output = ""
+	for c in braille_text:
+		if (ord(c) != 0) and (ord(c) != 65535):
+			output += c
+	braille_text = output
 
-        os.system(argv)
-
-        # Write braille output to text buffer.
-        braille_file = open(outfile)
-        braille_text = braille_file.read()
         self.buffer.set_text(braille_text)
-        braille_file.close()
 
         self.braille_buffer.set_text(
             ''.join([ascii_braille.ascii_to_braille.get(
                         x, '') for x in braille_text]))
+	return True
         
-        os.remove(config_fn)
-        os.remove(outfile)
